package com.factory.wx.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.entity.po.remote.UserInfo;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.mapper.UserInfoMapper;
import com.factory.web.entity.fa.*;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.fa.FirstArticleApprovalTaskMapper;
import com.factory.web.mapper.fa.FirstArticleCalendarMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalTaskMapper;
import com.factory.web.mapper.pm.OnsiteCalendarMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.fa.*;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.req.PlanPassReq;
import com.factory.wx.entity.res.PlanApprovalTaskRes;
import com.factory.wx.entity.res.PlanPassDetailRes;
import com.factory.wx.entity.res.PlanPassRes;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.PlanPassService;
import com.factory.wx.service.WxPlanService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Service("planPassService")
public class PlanPassServiceImpl implements PlanPassService {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanPassServiceImpl.class);
    /**
     * 审批规则类型为：1新建计划审批
     */
    private static int APPROVAL_TYPE_NEW = 1;
    /**
     * 审批规则类型为：2首件
     */
    private static int APPROVAL_TYPE_FIRST_ARTICLE = 2;
    /**
     * 审批规则类型为：3防错
     */
    private static int APPROVAL_TYPE_PM = 3;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteCalendarMapper onsiteCalendarMapper;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private OnsiteApprovalTaskMapper onsiteApprovalTaskMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private FirstArticleApprovalTaskMapper firstArticleApprovalTaskMapper;
    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;
    @Autowired
    private FirstArticleItemService firstArticleItemService;
    @Autowired
    private FirstArticleItemResultService firstArticleItemResultService;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private FirstArticleTaskService firstArticleTaskService;
    @Autowired
    private WxPlanService wxPlanService;

    private static IPage<BentelerPlan> getBentelerPlanIPage(PlanPageReq planPageReq, List<OnsiteCalendar> onsiteCalendarList, BentelerPlanMapper bentelerPlanMapper) {
        IPage<BentelerPlan> bentelerPlanIPage;
        List<Long> planIds = onsiteCalendarList.stream().map(OnsiteCalendar::getBentelerPlanId).collect(Collectors.toList());

        LambdaQueryWrapper<BentelerPlan> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(BentelerPlan::getId, planIds);
        wrapper.eq(BentelerPlan::getPlanType, planPageReq.getCheckType());

        if (StringUtils.isNotEmpty(planPageReq.getName())) {
            wrapper.like(BentelerPlan::getName, planPageReq.getName());
        }
        IPage<BentelerPlan> page = new Page<>(planPageReq.getPage(), planPageReq.getSize());
        bentelerPlanIPage = bentelerPlanMapper.selectPage(page, wrapper);
        return bentelerPlanIPage;
    }


    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public PageBean<BentelerPlan> getPassListByPage(PlanPageReq planPageReq) {

        LambdaQueryWrapper<OnsiteApprovalTask> approvalTaskWrapper = new LambdaQueryWrapper<>();
        approvalTaskWrapper.eq(OnsiteApprovalTask::getApprovalUserId, planPageReq.getUserId());
        approvalTaskWrapper.eq(OnsiteApprovalTask::getStatus, 0);//获取未处理过的计划

        List<OnsiteApprovalTask> onsiteApprovalTaskList = onsiteApprovalTaskMapper.selectList(approvalTaskWrapper);
        IPage<BentelerPlan> bentelerPlanIPage = new Page<>();
        if (CollUtil.isNotEmpty(onsiteApprovalTaskList)) {
            List<Long> taskIds = onsiteApprovalTaskList.stream().map(OnsiteApprovalTask::getOnsiteTaskId).collect(Collectors.toList());
            List<OnsiteTask> onsiteTaskList = onsiteTaskMapper.selectBatchIds(taskIds);
            if (CollUtil.isNotEmpty(onsiteTaskList)) {
                List<Long> calendarIds = onsiteTaskList.stream().map(OnsiteTask::getOnsiteCalendarId).collect(Collectors.toList());
                List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarMapper.selectBatchIds(calendarIds);
                if (CollUtil.isNotEmpty(onsiteCalendarList)) {
                    bentelerPlanIPage = getBentelerPlanIPage(planPageReq, onsiteCalendarList, bentelerPlanMapper);
                }
            }
        }
        return PageBean.<BentelerPlan>builder().row(bentelerPlanIPage.getRecords()).total(bentelerPlanIPage.getTotal()).page(bentelerPlanIPage.getPages()).build();
    }

    /**
     * 计划放行详情-首件
     *
     * @param planPassReq
     * @return
     */

    @Override
    public PlanPassDetailRes getPassDetail(PlanPassReq planPassReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes =
                wxPlanService.getFirstArticlePlanBaseInfo(planPassReq.getPlanId());

        //工作站
        List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, planPassReq.getPlanId()));
        // 每个workshop下的item
        for (int i = 0; i < firstArticleWorkshopList.size(); i++) {
            FirstArticleWorkshop workshop = firstArticleWorkshopList.get(i);
            List<FirstArticleItem> firstArticleItemList = firstArticleItemService.list(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, workshop.getId()));
            //每个item下的结果
            for (int j = 0; j < firstArticleItemList.size(); j++) {
                FirstArticleItem item = firstArticleItemList.get(j);
                //TODO 一个对应多个result
                List<FirstArticleItemResult> firstArticleItemResults = firstArticleItemResultService.list(Wrappers.<FirstArticleItemResult>lambdaQuery()
                        .eq(FirstArticleItemResult::getFirstArticleItemId, item.getId())
                        .eq(FirstArticleItemResult::getResult, 1));
                // 容错，没有result信息则删除
                if (firstArticleItemResults.stream().count() == 0) {
                    firstArticleItemList.remove(j);
                } else {
                    // 首件列表
                    if (item.getFirstFlag() == 1 && firstArticleItemResults.stream().count() > 0) {
                        item.setFirstItemResultList(firstArticleItemResults);
                    }
                    // 200件
                    if (item.getTwoHundredFlag() == 1 && firstArticleItemResults.stream().count() > 0) {
                        item.setTwoHundredItemResultList(firstArticleItemResults);
                    }
                    // 中断
                    if (item.getCatchFlag() == 1 && firstArticleItemResults.stream().count() > 0) {
                        item.setCatchItemResultList(firstArticleItemResults);
                    }
                    if (firstArticleItemResults.stream().count() > 0) {
                        Optional<Long> taskId = firstArticleItemResults.stream().findFirst().map(FirstArticleItemResult::getFirstArticleTaskId);
                        if (taskId.isPresent()) {
                            FirstArticleTask firstArticleTask = firstArticleTaskService.getById(taskId.get().longValue());
                            item.setFirstArticleTask(firstArticleTask);

                            FirstArticleApprovalTask approvalTask = firstArticleApprovalTaskMapper.selectOne(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                                    .eq(FirstArticleApprovalTask::getFirstArticleTaskId, taskId.get().longValue())
//                                    .eq(FirstArticleApprovalTask::getApprovalLevel, firstArticleApprovalTask.getApprovalLevel())
                                    .eq(FirstArticleApprovalTask::getApprovalUserId, planPassReq.getUserId()));
                            item.setFirstArticleApprovalTaskStatus(approvalTask.getStatus());
                        }
                    }
                }
            }
            //工作站下面没有异常的item则不显示工作站
            if (firstArticleItemList.size() > 0) {
                workshop.setFirstArticleItemList(firstArticleItemList);
            } else {
                firstArticleWorkshopList.remove(i);
            }
        }

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(wxPlanBaseInfoRes.getFrequency())
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .firstArticleTitle((FirstArticleTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .firstArticleWorkshops(firstArticleWorkshopList)
                .build();
    }

    /**
     * 计划放行详情-防错
     *
     * @param planPassReq
     * @return
     */
    @Override
    public PlanPassDetailRes getPassDetailPM(PlanPassReq planPassReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes =
                wxPlanService.getOnsitePlanBaseInfo(planPassReq.getPlanId());

        //获取待审批任务
        Map map = new HashMap();
        map.put("userId", planPassReq.getUserId());
        List<PlanApprovalTaskRes> planApprovalTaskResList = planPassMapper.getApprovalTaskList(map);

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(wxPlanBaseInfoRes.getFrequency())
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .planApprovalTasks(planApprovalTaskResList)
                .build();
    }

    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean approvalTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {

        int updateStatus = 0; // 更新状态
        int taskStatus = 0; //task表状态
        if (planApprovalTaskReq.getStatus() == 1) taskStatus = 2; // 放行
        if (planApprovalTaskReq.getStatus() == 2) taskStatus = 3; // 不通过

        if (planApprovalTaskReq.getPlanType() == 1) {
            //更新首件审批信息
            updateStatus = updateFirstArticleApprovalTask(planApprovalTaskReq, taskStatus);
        }
        if (planApprovalTaskReq.getPlanType() == 2) {
            //更新防错审批信息
            updateStatus = updateOnsiteApprovalTask(planApprovalTaskReq, taskStatus);
        }

        if (updateStatus > 0) {
            settingApprovalTask(planApprovalTaskReq);
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 更新首件任务审批状态
     *
     * @param planApprovalTaskReq
     * @param taskStatus
     * @return
     */
    private int updateFirstArticleApprovalTask(PlanApprovalTaskReq planApprovalTaskReq, int taskStatus) {
        int updateStatus;
        FirstArticleApprovalTask ot = new FirstArticleApprovalTask();
        ot.setFirstArticleTaskId(planApprovalTaskReq.getTaskId());
        ot.setReason(planApprovalTaskReq.getReason());
        ot.setStatus(planApprovalTaskReq.getStatus());
        updateStatus = firstArticleApprovalTaskMapper.update(ot, Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                .eq(FirstArticleApprovalTask::getFirstArticleTaskId, planApprovalTaskReq.getTaskId())
                .eq(FirstArticleApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        //更新task表状态
        firstArticleTaskService.updateById(FirstArticleTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
        return updateStatus;
    }

    /**
     * 更新防错任务审批状态
     *
     * @param planApprovalTaskReq
     * @param taskStatus
     * @return
     */
    private int updateOnsiteApprovalTask(PlanApprovalTaskReq planApprovalTaskReq, int taskStatus) {
        int updateStatus;
        OnsiteApprovalTask ot = new OnsiteApprovalTask();
        ot.setOnsiteTaskId(planApprovalTaskReq.getTaskId());
        ot.setReason(planApprovalTaskReq.getReason());
        ot.setStatus(planApprovalTaskReq.getStatus());
        updateStatus = onsiteApprovalTaskMapper.update(ot, Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        //更新task表状态
        onsiteTaskMapper.updateById(OnsiteTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
        return updateStatus;
    }

    /**
     * 设置审批task任务
     *
     * @param planApprovalTaskReq
     */
    private void settingApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        //审批不通过触发下一级别
        if (planApprovalTaskReq.getStatus() == 2) {
            //设置首件下一级别审批
            if (planApprovalTaskReq.getPlanType() == 1) {
                setNextApprovalLevel(planApprovalTaskReq);
            }
            //设置防错下一级别审批
            if (planApprovalTaskReq.getPlanType() == 2) {
                setNextPMApprovalLevel(planApprovalTaskReq);
            }
        }
        //审批通过，保存审批历史
        if (planApprovalTaskReq.getStatus() == 1) {
            //首件
            if (planApprovalTaskReq.getPlanType() == 1) {
                firstArticleService.setFirstArticleTaskHis(planApprovalTaskReq.getTaskId());
            }
            //防错
            if (planApprovalTaskReq.getPlanType() == 2) {
                preventMistakeService.setOnsiteTaskHis(planApprovalTaskReq.getTaskId());
            }
        }
    }

    /**
     * 设置防错下一级别人员
     *
     * @param planApprovalTaskReq
     */
    private void setNextPMApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
        ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                .eq(ApprovalRule::getType, APPROVAL_TYPE_PM)
                .orderByDesc(ApprovalRule::getApprovalLevel)
                .last("limit 1"));
        //获取当前审批级别
        OnsiteApprovalTask onsiteApprovalTask = onsiteApprovalTaskMapper.selectOne(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
        int userLevel = onsiteApprovalTask.getApprovalLevel();
        if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
            ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
            onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                    .onsiteTaskId(onsiteApprovalTask.getOnsiteTaskId())
                    .approvalLevel(approvalUserRes.getApprovalLevel())
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0)
                    .build());
        }
    }

    /**
     * 设置首件下一级别人员
     *
     * @param planApprovalTaskReq
     */
    private void setNextApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
        ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                .eq(ApprovalRule::getType, APPROVAL_TYPE_FIRST_ARTICLE)
                .orderByDesc(ApprovalRule::getApprovalLevel)
                .last("limit 1"));
        //获取当前审批级别
        FirstArticleApprovalTask firstArticleApprovalTask = firstArticleApprovalTaskMapper.selectOne(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                .eq(FirstArticleApprovalTask::getFirstArticleTaskId, planApprovalTaskReq.getTaskId())
                .eq(FirstArticleApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
        int userLevel = firstArticleApprovalTask.getApprovalLevel();
        if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
            ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
            onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                    .onsiteTaskId(firstArticleApprovalTask.getFirstArticleTaskId())
                    .approvalLevel(approvalUserRes.getApprovalLevel())
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0)
                    .build());
        }
    }


    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public List<PlanPassRes> getPassList(PlanPageReq planPageReq) {
        //分页
        planPageReq.setPage((planPageReq.getPage() - 1) * planPageReq.getSize());
        List<PlanPassRes> planPassResList = new ArrayList<>();
        //首件列表
        if (planPageReq.getCheckType() == 1) {
            planPassResList = planPassMapper.getPlanPassList(planPageReq);
        }
        //防错列表
        if (planPageReq.getCheckType() == 2) {
            planPassResList = planPassMapper.getPlanPassPMList(planPageReq);
        }
        return planPassResList;
    }

}
