package com.jihengbel.intelligent.applicationtask.init;

import com.jihengbel.intelligent.applicationtask.service.FirstTaskService;
import com.jihengbel.intelligent.applicationtask.service.OnsiteTaskService;
import org.quartz.Scheduler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Component;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * 这个是启动就触发的部分
 */
@Component
public class TimerTaskInit implements CommandLineRunner {

    @Value("${onsite.task.cron}")
    private String cron_1;
    @Value("${onsite.remind.work.cron}")
    private String cron_2;

    @Value("${first.task.cron}")
    private String cron_3;
    @Value("${first.remind.work.cron}")
    private String cron_4;

    @Autowired
    OnsiteTaskService onsiteTaskService;

    @Autowired
    FirstTaskService firstTaskService;

    @Autowired
    SchedulerFactoryBean schedulerFactoryBean;

    @Override
    public void run(String... args) throws Exception {
        ConcurrentMap<String, Object> param = new ConcurrentHashMap<String, Object>();
        param.put("onsiteTaskService", onsiteTaskService);
        param.put("firstTaskService", firstTaskService);
        Scheduler sche = schedulerFactoryBean.getScheduler();
        addAutoApproveErrorTask(param, sche);
        addOnsiteTask(param, sche);
        addOnsiteRemindWork( param, sche);
        addFirstTask(param, sche);
        addFirstRemindWork(param, sche);
    }

    /**
     * 添加异常自动审批
     *
     * @param param
     * @param sche
     */
    private void addAutoApproveErrorTask(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("errAutoTask");
        onsite.setParamMap(param);
        onsite.setTriggerRule("0 */5 * * * ? ");
        onsite.setClazz(AutoApproveErrorTaskJob.class);
        QuartzManager.addJob(sche, onsite);
    }

    /**
     * 添加onsit任务的定时器
     *
     * @param param
     * @param sche
     */
    private void addOnsiteTask(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("onsiteTask");
        onsite.setParamMap(param);
        onsite.setTriggerRule(cron_1);//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(OnsiteTaskJob.class);
        QuartzManager.addJob(sche, onsite);
    }

    private void addFirstTask(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("firstTask");
        onsite.setParamMap(param);
        onsite.setTriggerRule(cron_3);//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(FirstTaskJob.class);
        QuartzManager.addJob(sche, onsite);
    }


    /**
     * 添加onsit工作提醒
     *
     * @param param
     * @param sche
     */
    private void addOnsiteRemindWork(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("onsiteRemindWork");
        onsite.setParamMap(param);
        onsite.setTriggerRule(cron_2);//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(OnsiteRemindJob.class);
        QuartzManager.addJob(sche, onsite);
    }

    private void addFirstRemindWork(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("firstRemindWork");
        onsite.setParamMap(param);
        onsite.setTriggerRule(cron_4);//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(FirstRemindJob.class);
        QuartzManager.addJob(sche, onsite);
    }

}
